package cn.com.qmth.examcloud.web.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.servlet.HandlerInterceptor;

import com.codahale.metrics.Meter;
import com.codahale.metrics.MetricRegistry;
import com.codahale.metrics.Timer;
import com.codahale.metrics.Timer.Context;

import cn.com.qmth.examcloud.web.actuator.MetricNames;
import cn.com.qmth.examcloud.web.actuator.MetricRegistryHolder;
import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.support.ApiInfo;

/**
 * API统计拦截器
 *
 * @author WANGWEI
 * @date 2019年7月24日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class ApiStatisticInterceptor implements HandlerInterceptor {

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {

		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

		if (null == apiInfo) {
			return true;
		}

		Timer timer = MetricRegistryHolder.getDefalut()
				.timer(MetricRegistry.name(MetricNames.API_TIMER.name(), apiInfo.getMapping()));
		Context ctx = timer.time();
		request.setAttribute(HttpServletRequestAttribute.$_METRICS_TIMER_CTX.name(), ctx);

		return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {
		Context ctx = (Context) request
				.getAttribute(HttpServletRequestAttribute.$_METRICS_TIMER_CTX.name());
		if (null != ctx) {
			ctx.stop();
		}
		Boolean hasException = (Boolean) request
				.getAttribute(HttpServletRequestAttribute.$_EXCEPTION_HAPPENED.name());

		if (null != hasException && hasException) {
			ApiInfo apiInfo = (ApiInfo) request
					.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

			if (null != apiInfo) {
				Meter meter = MetricRegistryHolder.getDefalut().meter(MetricRegistry
						.name(MetricNames.API_EXCEPTION_METER.name(), apiInfo.getMapping()));
				meter.mark();
			}
		}

	}
}
